package archiveModels

//go:generate reform --gofmt=false

import (
	"database/sql"
	asuModels "devel.mephi.ru/dyokunev/go-asu-models"
	I "devel.mephi.ru/dyokunev/wwwvoip/app/iface"
	"encoding/json"
	"github.com/revel/revel"
	"github.com/xaionaro-go/extime"
	"strconv"
	"strings"
	"voip/app/helpers"
)

//reform:phone
type Phone struct {
	Id                    int                  `json:"id" reform:"id,pk" view:"omit"`
	MAC                   string               `json:"mac" reform:"mac" sql:"index"`
	Type                  string               `json:"type" reform:"type"`
	Model                 string               `json:"model" reform:"model"`
	SubscriberKey         *int64               `json:"subscriber_key" reform:"subscriber_key"`
	Subscriber            I.UserBase           `json:"-" reform:"-"`
	PinnedToUnitArchiveId *int                 `json:"pinned_to_unit_archive_id" reform:"pinned_to_unit_archive_id" view:"omit"`
	PinnedToUnitArchive   asuModels.Unit       `json:"-" reform:"-"`
	Lines                 []string             `json:"lines" reform:"-"`
	LinesStr              string               `json:"-" reform:"lines" view:"omit"`
	Notes                 string               `json:"notes" reform:"notes" sql:"size:1024"`
	StfCardIds            []int                `json:"stf_card_ids" reform:"-" view:"omit"`
	StfCardIdsStr         string               `json:"-" reform:"stf_card_ids" view:"omit"`
	Formulars             []asuModels.Formular `json:"formulars" reform:"-"`
	CreatedAt             extime.Date          `json:"created_at" reform:"created_at"`
	IsActual              bool                 `json:"is_actual" reform:"is_actual" sql:"index"`
}

func (p *Phone) AfterFind() error {
	p.StfCardIds = []int{}
	stfCardIds := strings.Split(p.StfCardIdsStr, ",")
	for _, stfCardIdStr := range stfCardIds {
		if len(stfCardIdStr) == 0 {
			continue
		}
		stfCardId, err := strconv.Atoi(stfCardIdStr)
		if err != nil {
			revel.ERROR.Printf(`Cannot convert "%v" to integer: %v`, stfCardIdStr, err.Error())
			continue
		}
		p.StfCardIds = append(p.StfCardIds, stfCardId)
	}

	p.Lines = helpers.RemoveEmpty(strings.Split(p.LinesStr, ",")).([]string)

	return nil
}

func (p *Phone) BeforeInsert() error {
	stfCardIds := []string{}
	for _, stfCardId := range p.StfCardIds {
		stfCardIds = append(stfCardIds, strconv.Itoa(stfCardId))
	}
	p.StfCardIdsStr = strings.Join(stfCardIds, ",")
	p.LinesStr = strings.Join(p.Lines, ",")

	p.CreatedAt = extime.Date(extime.Now())

	return nil
}

func (p *Phone) PrepareFormulars() *Phone {
	if len(p.StfCardIds) == 0 {
		return p
	}

	var err error
	p.Formulars, err = asuModels.FormularSQL.Select(`stfcardid IN (?)`, p.StfCardIds)
	if err != nil {
		revel.ERROR.Printf("Got error: %v", err.Error())
	}
	return p
}

func (p *Phone) PreparePinnedToUnitArchive() *Phone {
	if p.PinnedToUnitArchiveId == nil {
		return p
	}

	var err error
	p.PinnedToUnitArchive, err = asuModels.UnitSQL.First(`id = ?`, *p.PinnedToUnitArchiveId)
	if err != nil {
		revel.ERROR.Printf("Got error: %v", err.Error())
	}
	return p
}

func (p Phone) UniqueHash() string {
	p.Id = 0
	p.IsActual = true
	p.CreatedAt = extime.Date{}
	p.Formulars = []asuModels.Formular{}
	if p.StfCardIds == nil {
		p.StfCardIds = []int{}
	}
	if p.Lines == nil {
		p.Lines = []string{}
	}

	b, err := json.Marshal(p)
	if err != nil {
		revel.ERROR.Panicf("Cannot JSONize an archive phone (%v): %v", p, err.Error())
	}

	return string(b)
}

func (t phoneTableTypeType) DistinctMACs() (distinctMacs []string, err error) {
	var rows *sql.Rows
	rows, err = PhoneSQL.SelectRows("DISTINCT `mac`")
	if err != nil {
		return
	}
	defer rows.Close()

	for rows.Next() {
		var mac string
		err = rows.Scan(&mac)
		if err != nil {
			return
		}
		distinctMacs = append(distinctMacs, mac)
	}
	err = rows.Err()

	return
}
